(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.Interactor = {}));
})(this, (function (exports) { 'use strict';

  const SelectorRE = /^(?:(?:"(?<str>.*)"\s*)?(?:\$\((?<sel>.*)\)|::(?<test>(?!\()\S+?)?(?:\((?<tval>.*)\))?)?|(?<text>(?!\$|:|").*))$/;
  const TestAttrOpRE = /^(?:([~|^$*])?=?(?:(["']?)(.*)\2))?/;
  function selectRelatedElement($) {
    if ($.tagName.toLowerCase() !== 'input') {
      let label = 'control' in $ ? $ : $.closest('label');
      if (label === $ || label?.innerText.startsWith($.innerText)) return label.control ?? label;
    }
    return $;
  }
  function createSelectorFunction(context, selector) {
    if (!selector) return context.selector;
    let ctx = {
      root: context.root,
      selector: context.selector
    };
    let selectorFn = typeof selector === 'function' ? selector : null;
    let parsed = typeof selector === 'string' ? SelectorRE.exec(selector)?.groups : {};
    if (!parsed) throw new Error(`Invalid selector: ${selector}`);
    let {
      str,
      sel,
      test,
      tval,
      text = str
    } = parsed;
    test &&= `-${test}`;
    tval &&= tval.replace(TestAttrOpRE, '$1="$3"');
    if (test || tval) sel = `[data-test${test ?? ''}${tval ?? ''}]`;
    if (text && !str) selector = `"${text}"`;
    let traverseDOM = _ref => {
      let {
        all,
        root,
        $
      } = _ref;
      $ = $.firstElementChild || function next($) {
        if (!$ || $ === root) return null;
        if ($.nextElementSibling) return $.nextElementSibling;
        return next($.parentElement);
      }($);
      if (all === true) all = [];
      if (!$) return all || undefined;
      let match = $ !== root && (!sel || $.matches(sel)) && (!text || $.innerText === text || $.placeholder === text);
      if (match && !all) return $;
      if (match && all) all = all.concat($);
      return traverseDOM({
        all,
        root,
        $
      });
    };
    let toString = () => {
      if (selector instanceof context.root().ownerDocument.defaultView.HTMLElement) return `${selector.tagName?.toLowerCase()} element`;
      if (typeof selector !== 'function' || selector.toString !== Function.toString) return selector.toString();
      return 'element';
    };
    return Object.assign(function select() {
      let {
        all = false
      } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      let root = ctx.root();
      let result = selector instanceof root.ownerDocument.defaultView.HTMLElement ? selector : (selectorFn ?? traverseDOM)({
        all,
        root,
        $: ctx.selector()
      });
      if (!result) throw new Error(`Could not find ${toString()}`);
      return typeof selector !== 'string' || sel || !result ? result : all ? [].concat(result).map(selectRelatedElement) : selectRelatedElement(result);
    }, {
      toString
    });
  }

  class Context {
    static Symbol = Symbol('@@interactor|context');
    constructor(i, options) {
      if (i instanceof Context) {
        return Object.assign(Object.setPrototypeOf(this, i), {
          assert: Object.setPrototypeOf({}, i.assert),
          immediate: i
        });
      }
      let {
        root,
        ...rest
      } = options ?? {};
      Object.defineProperty(this, 'i', {
        value: this.i = i
      });
      Object.defineProperty(this, 'root', {
        value: this.root = Object.assign(() => typeof root !== 'function' ? root ?? globalThis.document.body : root(), {
          toString: () => 'root element'
        })
      });
      this.assert = {
        timeout: 1000,
        frequency: 60,
        reliability: 1
      };
      this.immediate = this;
      this.set(rest);
    }
    get selector() {
      return Object.assign(options => options?.all ? [this.root()] : this.root(), {
        toString: this.root.toString
      });
    }
    set selector(selector) {
      let {
        set
      } = Object.getOwnPropertyDescriptor(this.constructor.prototype, 'selector');
      let selectorFn = createSelectorFunction(this, selector);
      Object.defineProperty(this, 'selector', {
        configurable: true,
        get: () => selectorFn,
        set
      });
    }
    get $() {
      return this.selector();
    }
    get $$() {
      return this.selector({
        all: true
      });
    }
    get set() {
      return _ref => {
        let {
          immediate,
          assert,
          ...rest
        } = _ref;
        if (immediate) Object.assign(this.immediate, immediate);
        if (assert) Object.assign(this.assert, assert);
        return Object.assign(this, rest);
      };
    }
  }

  function* assert(assertion, failureMessage, negatedMessage) {
    let expected = yield _ref => {
      let {
        assert
      } = _ref;
      return assert?.expected;
    };
    let error, pass;
    try {
      pass = yield assertion;
      if (pass === undefined) pass = true;
    } catch (e) {
      if (expected) error = e;
      pass = false;
    }
    if (!!pass !== expected) {
      throw error ?? new Error((expected ? (yield failureMessage) || 'Assertion failed' : (yield negatedMessage) || 'Expected assertion to fail').replace('#{this}', yield c => `${c.selector}`));
    }
    return true;
  }
  class Assertion {
    static Symbol = Symbol('@@interactor|assertion');
    constructor(assertion, failureMessage, negatedMessage) {
      if (typeof assertion === 'object' && Object.getPrototypeOf(assertion) === Object.prototype) ({
        assertion,
        failureMessage,
        negatedMessage
      } = assertion);
      Object.defineProperty(this, Assertion.Symbol, {
        value: () => assert(assertion, failureMessage, negatedMessage),
        enumerable: false
      });
      Object.defineProperty(this, Context.Symbol, {
        value: {
          assert: {
            expected: true
          }
        },
        enumerable: false
      });
    }
  }

  const Chain = Symbol('@@interactor|chain');
  function flattenChain(chain) {
    return [chain].reduce(function reduce(chain, next) {
      if (next?.[Chain]) chain.unshift(next?.[Chain]);
      return !next ? chain : reduce(chain, Object.getPrototypeOf(next));
    }, []);
  }
  function chainable(value, instance, handler) {
    let next = Object.setPrototypeOf({}, typeof instance === 'function' ? Object.getPrototypeOf(instance) : instance);
    Object.defineProperty(next, Chain, {
      value,
      enumerable: false
    });
    return Object.setPrototypeOf({
      then: Object.setPrototypeOf(function then(onFulfilled, onRejected) {
        let signal = onFulfilled?.signal;
        if (signal) onFulfilled = v => v;
        while (instance[Chain]) instance = Object.getPrototypeOf(instance);
        return handler(instance, flattenChain(next), signal).then(onFulfilled, onRejected);
      }, next)
    }, value);
  }

  class Interaction {
    static Symbol = Symbol('@@interactor|interaction');
    constructor(interaction) {
      Object.defineProperty(this, Interaction.Symbol, {
        *value() {
          return interaction;
        },
        enumerable: false
      });
      Object.defineProperty(this, Context.Symbol, {
        enumerable: false,
        value: {}
      });
    }
  }

  function waitForTimeout(timeout, value) {
    return new Promise(r => setTimeout(r, timeout, value));
  }
  async function* retry(fn) {
    let {
      timeout,
      frequency,
      reliability: rel
    } = yield _ref => {
      let {
        assert
      } = _ref;
      return assert;
    };
    let error = new Error('Timeout exceeded');
    let interval = 1000 / frequency;
    let start = Date.now();
    let result;
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }
    while (true) {
      if (Date.now() - start >= timeout) throw error;
      try {
        result = yield fn(...args);
      } catch (e) {
        error = e;
      }
      if (!result) await waitForTimeout(interval);else if (rel) await waitForTimeout(interval, rel -= 1);else return result;
    }
  }

  async function iterate(iter, callback, signal) {
    let done, value, error;
    while (!done) {
      try {
        value = await callback(value);
      } catch (e) {
        error = e;
      }
      ({
        done,
        value
      } = error ?? signal?.aborted ? await iter.throw(error ?? signal.reason) : await iter.next(value));
    }
    return callback(value);
  }
  function interact(context, subject, signal) {
    context = new Context(context[Context.Symbol] ?? context);
    let handle = async value => {
      if (value?.then?.[Chain]) {
        return flattenChain(value.then).slice(0, -1).reduce((promise, next) => {
          return promise.then(() => handle(next));
        }, Promise.resolve());
      }
      while (value && (typeof value[Interaction.Symbol] === 'function' || typeof value[Assertion.Symbol] === 'function' || typeof value.next === 'function' || typeof value.then === 'function' || typeof value === 'function')) {
        if (typeof value[Interaction.Symbol] === 'function' || typeof value[Assertion.Symbol] === 'function' && context.assert.expected != null) value = interact(context, value, signal);else if (typeof value[Assertion.Symbol] === 'function') value = retry(interact, context, value, signal);else if (typeof value === 'function') value = value.call(context.i, context);else if (typeof value.next === 'function') value = iterate(value, handle, signal);else value = await value;
      }
      return value;
    };
    if (subject[Context.Symbol]) context.set(subject[Context.Symbol]);
    if (subject[Interaction.Symbol]) return handle(subject[Interaction.Symbol]);
    if (subject[Assertion.Symbol]) return handle(subject[Assertion.Symbol]);
    return [].concat(subject).reduce((promise, value) => {
      return promise.then(() => handle(value));
    }, Promise.resolve());
  }

  class Assert {
    static Symbol = Symbol('@@interactor|assert');
    static defineAssertion(name, assertion, failureMessage, negatedMessage) {
      if (typeof assertion === 'object' && Object.getPrototypeOf(assertion) === Object.prototype) ({
        assertion,
        failureMessage,
        negatedMessage
      } = assertion);
      Object.defineProperty(this.prototype, name, {
        configurable: true,
        value: Object.defineProperty(function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }
          let {
            i,
            ...context
          } = this[Assert.Symbol];
          let assert = assertion.prototype instanceof Assertion ? new assertion(...args) : new i.constructor.Assertion(function () {
            return assertion.apply(this, args);
          }, failureMessage, negatedMessage);
          Object.assign(assert[Context.Symbol], context);
          return chainable(assert, i, interact);
        }, 'name', {
          value: name
        })
      });
      return this;
    }
    static defineAssertions(assertions) {
      for (let name in assertions) this.defineAssertion(name, assertions[name]);
      return this;
    }
    constructor(i, context) {
      Object.defineProperty(this, Assert.Symbol, {
        value: {
          i,
          ...context,
          assert: {
            expected: true,
            ...context?.assert
          }
        },
        enumerable: false
      });
    }
    get not() {
      let {
        i,
        assert,
        ...context
      } = this[Assert.Symbol];
      if (!assert.expected) throw new Error('Double negative');
      context = {
        ...context,
        assert: {
          expected: false,
          ...context.assert
        }
      };
      return Object.setPrototypeOf(Object.defineProperty(typeof this === 'function' ? function not(assertion, negatedMessage) {
        let assert = !(assertion instanceof Assertion) ? new i.constructor.Assertion(assertion, null, negatedMessage) : assertion;
        return chainable(Object.setPrototypeOf({
          [Context.Symbol]: {
            ...assert[Context.Symbol],
            ...context
          }
        }, assert), i, interact);
      } : {}, Assert.Symbol, {
        value: {
          i,
          ...context
        },
        enumerable: false
      }), this);
    }
  }

  class Interactor {
    static Assert = Assert;
    static Assertion = Assertion;
    static Context = Context;
    static Interaction = Interaction;
    static defineAction(name, action) {
      Object.defineProperty(this.prototype, name, {
        configurable: true,
        value: Object.defineProperty(function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }
          return chainable(action.prototype instanceof this.constructor.Interaction ? new action(...args) : new this.constructor.Interaction(function () {
            return action.apply(this, args);
          }), this, interact);
        }, 'name', {
          value: name
        })
      });
      return this;
    }
    static defineActions(interactions) {
      for (let name in interactions) this.defineAction(name, interactions[name]);
      return this;
    }
    static defineAssertion(name, assertion, failureMessage, negatedMessage) {
      this.Assert.defineAssertion(name, assertion, failureMessage, negatedMessage);
      return this;
    }
    static defineAssertions(assertions) {
      this.Assert.defineAssertions(assertions);
      return this;
    }
    constructor(options) {
      Object.defineProperty(this, Context.Symbol, {
        value: new this.constructor.Context(this, options),
        enumerable: false
      });
    }
    act(interaction) {
      return chainable(!(interaction instanceof this.constructor.Interaction) ? new this.constructor.Interaction(interaction) : interaction, this, interact);
    }
    get assert() {
      return Object.setPrototypeOf(function assert(assertion, failureMessage) {
        return chainable(!(assertion instanceof this.constructor.Assertion) ? new this.constructor.Assertion(assertion, failureMessage) : assertion, this, interact);
      }, new this.constructor.Assert(this));
    }
    find(selector) {
      let {
        then: scoped
      } = chainable(Object.defineProperty({}, Interaction.Symbol, {
        enumerable: false,
        value: function find(_ref) {
          let {
            set
          } = _ref;
          if (selector) set({
            immediate: {
              selector
            }
          });
        }
      }), this, interact);
      return chainable(Object.setPrototypeOf(Object.defineProperty({
        times(number) {
          let {
            i,
            ...context
          } = this[Assert.Symbol];
          let assert = new i.constructor.Assertion(_ref2 => {
            let {
              $$
            } = _ref2;
            return $$.length === number;
          }, `Expected to find #{this} ${number} times`, `Expected not to find #{this} ${number} times`);
          Object.assign(assert[Context.Symbol], context);
          return chainable(assert, scoped, interact);
        }
      }, Interaction.Symbol, {
        enumerable: false,
        value: function* exists() {
          yield this.assert.exists();
          return _ref3 => {
            let {
              $
            } = _ref3;
            return $;
          };
        }
      }), new this.constructor.Assert(scoped)), scoped, interact);
    }
  }

  function* blur(selector) {
    yield this.find(selector);
    yield this.assert.focused();
    yield _ref => {
      let {
        $
      } = _ref;
      return $.blur();
    };
  }

  function* click(selector) {
    yield this.find(selector);
    yield this.assert.not.disabled();
    let $option = yield _ref => {
      let {
        $
      } = _ref;
      return $.tagName.toLowerCase() === 'option' ? $ : $.closest('option');
    };
    let $select = $option?.closest('select');
    if ($select) {
      yield this.assert(!$select.disabled, '#{this} select is disabled');
      $option.selected = $select.multiple ? !$option.selected : true;
      yield this.find($select);
      yield this.trigger('input', {
        cancelable: false
      });
      yield this.trigger('change', {
        cancelable: false
      });
    } else {
      yield _ref2 => {
        let {
          $
        } = _ref2;
        return $.click();
      };
    }
  }

  function* focus(selector) {
    yield this.find(selector);
    yield this.assert.focusable();
    yield _ref => {
      let {
        $
      } = _ref;
      return $.focus();
    };
  }

  class Keyboard {
    constructor(previous) {
      this.state = [...(previous?.state ?? [])];
    }
    get(key, canonical) {
      if (!Keyboard.Events[key]) throw new Error(`Unknown key \`${key}\``);
      let k = Keyboard.Events[key];
      return canonical && 'code' in k ? Keyboard.Events[k.code] : k;
    }
    pressed(key) {
      return !!Keyboard.Events[key] && this.state.includes(this.get(key, true));
    }
    toggle(key) {
      let k = this.get(key, true);
      let existing = this.state.indexOf(k);
      if (~existing) this.state.splice(existing, 1);else this.state.push(k);
      return this;
    }
    parse(key) {
      let k = this.get(key);
      let altKey = this.pressed('Alt');
      let ctrlKey = this.pressed('Control');
      let metaKey = this.pressed('Meta');
      let shiftKey = this.pressed('Shift');
      return {
        text: !(altKey || ctrlKey || metaKey) && (k.key.length === 1 ? shiftKey && k.shiftKey || k.key : k.text) || '',
        event: {
          code: k.code,
          location: k.location || 0,
          repeat: this.pressed(key) || false,
          key: shiftKey && k.shiftKey || k.key,
          keyCode: shiftKey && k.shiftKeyCode || k.keyCode,
          altKey,
          ctrlKey,
          metaKey,
          shiftKey
        }
      };
    }
  }
  function freeze(object) {
    if (object && typeof object === 'object' && !Object.isFrozen(object)) {
      for (let k of Reflect.ownKeys(object)) freeze(object[k]);
      return Object.freeze(object);
    } else {
      return object;
    }
  }
  Keyboard.Events = freeze({
    '0': {
      keyCode: 48,
      key: '0',
      code: 'Digit0'
    },
    '1': {
      keyCode: 49,
      key: '1',
      code: 'Digit1'
    },
    '2': {
      keyCode: 50,
      key: '2',
      code: 'Digit2'
    },
    '3': {
      keyCode: 51,
      key: '3',
      code: 'Digit3'
    },
    '4': {
      keyCode: 52,
      key: '4',
      code: 'Digit4'
    },
    '5': {
      keyCode: 53,
      key: '5',
      code: 'Digit5'
    },
    '6': {
      keyCode: 54,
      key: '6',
      code: 'Digit6'
    },
    '7': {
      keyCode: 55,
      key: '7',
      code: 'Digit7'
    },
    '8': {
      keyCode: 56,
      key: '8',
      code: 'Digit8'
    },
    '9': {
      keyCode: 57,
      key: '9',
      code: 'Digit9'
    },
    'Power': {
      key: 'Power',
      code: 'Power'
    },
    'Eject': {
      key: 'Eject',
      code: 'Eject'
    },
    'Abort': {
      keyCode: 3,
      code: 'Abort',
      key: 'Cancel'
    },
    'Help': {
      keyCode: 6,
      code: 'Help',
      key: 'Help'
    },
    'Backspace': {
      keyCode: 8,
      code: 'Backspace',
      key: 'Backspace'
    },
    'Tab': {
      keyCode: 9,
      code: 'Tab',
      key: 'Tab'
    },
    'Numpad5': {
      keyCode: 12,
      shiftKeyCode: 101,
      key: 'Clear',
      code: 'Numpad5',
      shiftKey: '5',
      location: 3
    },
    'NumpadEnter': {
      keyCode: 13,
      code: 'NumpadEnter',
      key: 'Enter',
      text: '\r',
      location: 3
    },
    'Enter': {
      keyCode: 13,
      code: 'Enter',
      key: 'Enter',
      text: '\r'
    },
    '\r': {
      keyCode: 13,
      code: 'Enter',
      key: 'Enter',
      text: '\r'
    },
    '\n': {
      keyCode: 13,
      code: 'Enter',
      key: 'Enter',
      text: '\r'
    },
    'ShiftLeft': {
      keyCode: 16,
      code: 'ShiftLeft',
      key: 'Shift',
      location: 1
    },
    'ShiftRight': {
      keyCode: 16,
      code: 'ShiftRight',
      key: 'Shift',
      location: 2
    },
    'ControlLeft': {
      keyCode: 17,
      code: 'ControlLeft',
      key: 'Control',
      location: 1
    },
    'ControlRight': {
      keyCode: 17,
      code: 'ControlRight',
      key: 'Control',
      location: 2
    },
    'AltLeft': {
      keyCode: 18,
      code: 'AltLeft',
      key: 'Alt',
      location: 1
    },
    'AltRight': {
      keyCode: 18,
      code: 'AltRight',
      key: 'Alt',
      location: 2
    },
    'Pause': {
      keyCode: 19,
      code: 'Pause',
      key: 'Pause'
    },
    'CapsLock': {
      keyCode: 20,
      code: 'CapsLock',
      key: 'CapsLock'
    },
    'Escape': {
      keyCode: 27,
      code: 'Escape',
      key: 'Escape'
    },
    'Convert': {
      keyCode: 28,
      code: 'Convert',
      key: 'Convert'
    },
    'NonConvert': {
      keyCode: 29,
      code: 'NonConvert',
      key: 'NonConvert'
    },
    'Space': {
      keyCode: 32,
      code: 'Space',
      key: ' '
    },
    'Numpad9': {
      keyCode: 33,
      shiftKeyCode: 105,
      key: 'PageUp',
      code: 'Numpad9',
      shiftKey: '9',
      location: 3
    },
    'PageUp': {
      keyCode: 33,
      code: 'PageUp',
      key: 'PageUp'
    },
    'Numpad3': {
      keyCode: 34,
      shiftKeyCode: 99,
      key: 'PageDown',
      code: 'Numpad3',
      shiftKey: '3',
      location: 3
    },
    'PageDown': {
      keyCode: 34,
      code: 'PageDown',
      key: 'PageDown'
    },
    'End': {
      keyCode: 35,
      code: 'End',
      key: 'End'
    },
    'Numpad1': {
      keyCode: 35,
      shiftKeyCode: 97,
      key: 'End',
      code: 'Numpad1',
      shiftKey: '1',
      location: 3
    },
    'Home': {
      keyCode: 36,
      code: 'Home',
      key: 'Home'
    },
    'Numpad7': {
      keyCode: 36,
      shiftKeyCode: 103,
      key: 'Home',
      code: 'Numpad7',
      shiftKey: '7',
      location: 3
    },
    'ArrowLeft': {
      keyCode: 37,
      code: 'ArrowLeft',
      key: 'ArrowLeft'
    },
    'Numpad4': {
      keyCode: 37,
      shiftKeyCode: 100,
      key: 'ArrowLeft',
      code: 'Numpad4',
      shiftKey: '4',
      location: 3
    },
    'Numpad8': {
      keyCode: 38,
      shiftKeyCode: 104,
      key: 'ArrowUp',
      code: 'Numpad8',
      shiftKey: '8',
      location: 3
    },
    'ArrowUp': {
      keyCode: 38,
      code: 'ArrowUp',
      key: 'ArrowUp'
    },
    'ArrowRight': {
      keyCode: 39,
      code: 'ArrowRight',
      key: 'ArrowRight'
    },
    'Numpad6': {
      keyCode: 39,
      shiftKeyCode: 102,
      key: 'ArrowRight',
      code: 'Numpad6',
      shiftKey: '6',
      location: 3
    },
    'Numpad2': {
      keyCode: 40,
      shiftKeyCode: 98,
      key: 'ArrowDown',
      code: 'Numpad2',
      shiftKey: '2',
      location: 3
    },
    'ArrowDown': {
      keyCode: 40,
      code: 'ArrowDown',
      key: 'ArrowDown'
    },
    'Select': {
      keyCode: 41,
      code: 'Select',
      key: 'Select'
    },
    'Open': {
      keyCode: 43,
      code: 'Open',
      key: 'Execute'
    },
    'PrintScreen': {
      keyCode: 44,
      code: 'PrintScreen',
      key: 'PrintScreen'
    },
    'Insert': {
      keyCode: 45,
      code: 'Insert',
      key: 'Insert'
    },
    'Numpad0': {
      keyCode: 45,
      shiftKeyCode: 96,
      key: 'Insert',
      code: 'Numpad0',
      shiftKey: '0',
      location: 3
    },
    'Delete': {
      keyCode: 46,
      code: 'Delete',
      key: 'Delete'
    },
    'NumpadDecimal': {
      keyCode: 46,
      shiftKeyCode: 110,
      code: 'NumpadDecimal',
      key: '\u0000',
      shiftKey: '.',
      location: 3
    },
    'Digit0': {
      keyCode: 48,
      code: 'Digit0',
      shiftKey: ')',
      key: '0'
    },
    'Digit1': {
      keyCode: 49,
      code: 'Digit1',
      shiftKey: '!',
      key: '1'
    },
    'Digit2': {
      keyCode: 50,
      code: 'Digit2',
      shiftKey: '@',
      key: '2'
    },
    'Digit3': {
      keyCode: 51,
      code: 'Digit3',
      shiftKey: '#',
      key: '3'
    },
    'Digit4': {
      keyCode: 52,
      code: 'Digit4',
      shiftKey: '$',
      key: '4'
    },
    'Digit5': {
      keyCode: 53,
      code: 'Digit5',
      shiftKey: '%',
      key: '5'
    },
    'Digit6': {
      keyCode: 54,
      code: 'Digit6',
      shiftKey: '^',
      key: '6'
    },
    'Digit7': {
      keyCode: 55,
      code: 'Digit7',
      shiftKey: '&',
      key: '7'
    },
    'Digit8': {
      keyCode: 56,
      code: 'Digit8',
      shiftKey: '*',
      key: '8'
    },
    'Digit9': {
      keyCode: 57,
      code: 'Digit9',
      shiftKey: '(',
      key: '9'
    },
    'KeyA': {
      keyCode: 65,
      code: 'KeyA',
      shiftKey: 'A',
      key: 'a'
    },
    'KeyB': {
      keyCode: 66,
      code: 'KeyB',
      shiftKey: 'B',
      key: 'b'
    },
    'KeyC': {
      keyCode: 67,
      code: 'KeyC',
      shiftKey: 'C',
      key: 'c'
    },
    'KeyD': {
      keyCode: 68,
      code: 'KeyD',
      shiftKey: 'D',
      key: 'd'
    },
    'KeyE': {
      keyCode: 69,
      code: 'KeyE',
      shiftKey: 'E',
      key: 'e'
    },
    'KeyF': {
      keyCode: 70,
      code: 'KeyF',
      shiftKey: 'F',
      key: 'f'
    },
    'KeyG': {
      keyCode: 71,
      code: 'KeyG',
      shiftKey: 'G',
      key: 'g'
    },
    'KeyH': {
      keyCode: 72,
      code: 'KeyH',
      shiftKey: 'H',
      key: 'h'
    },
    'KeyI': {
      keyCode: 73,
      code: 'KeyI',
      shiftKey: 'I',
      key: 'i'
    },
    'KeyJ': {
      keyCode: 74,
      code: 'KeyJ',
      shiftKey: 'J',
      key: 'j'
    },
    'KeyK': {
      keyCode: 75,
      code: 'KeyK',
      shiftKey: 'K',
      key: 'k'
    },
    'KeyL': {
      keyCode: 76,
      code: 'KeyL',
      shiftKey: 'L',
      key: 'l'
    },
    'KeyM': {
      keyCode: 77,
      code: 'KeyM',
      shiftKey: 'M',
      key: 'm'
    },
    'KeyN': {
      keyCode: 78,
      code: 'KeyN',
      shiftKey: 'N',
      key: 'n'
    },
    'KeyO': {
      keyCode: 79,
      code: 'KeyO',
      shiftKey: 'O',
      key: 'o'
    },
    'KeyP': {
      keyCode: 80,
      code: 'KeyP',
      shiftKey: 'P',
      key: 'p'
    },
    'KeyQ': {
      keyCode: 81,
      code: 'KeyQ',
      shiftKey: 'Q',
      key: 'q'
    },
    'KeyR': {
      keyCode: 82,
      code: 'KeyR',
      shiftKey: 'R',
      key: 'r'
    },
    'KeyS': {
      keyCode: 83,
      code: 'KeyS',
      shiftKey: 'S',
      key: 's'
    },
    'KeyT': {
      keyCode: 84,
      code: 'KeyT',
      shiftKey: 'T',
      key: 't'
    },
    'KeyU': {
      keyCode: 85,
      code: 'KeyU',
      shiftKey: 'U',
      key: 'u'
    },
    'KeyV': {
      keyCode: 86,
      code: 'KeyV',
      shiftKey: 'V',
      key: 'v'
    },
    'KeyW': {
      keyCode: 87,
      code: 'KeyW',
      shiftKey: 'W',
      key: 'w'
    },
    'KeyX': {
      keyCode: 88,
      code: 'KeyX',
      shiftKey: 'X',
      key: 'x'
    },
    'KeyY': {
      keyCode: 89,
      code: 'KeyY',
      shiftKey: 'Y',
      key: 'y'
    },
    'KeyZ': {
      keyCode: 90,
      code: 'KeyZ',
      shiftKey: 'Z',
      key: 'z'
    },
    'MetaLeft': {
      keyCode: 91,
      code: 'MetaLeft',
      key: 'Meta',
      location: 1
    },
    'MetaRight': {
      keyCode: 92,
      code: 'MetaRight',
      key: 'Meta',
      location: 2
    },
    'ContextMenu': {
      keyCode: 93,
      code: 'ContextMenu',
      key: 'ContextMenu'
    },
    'NumpadMultiply': {
      keyCode: 106,
      code: 'NumpadMultiply',
      key: '*',
      location: 3
    },
    'NumpadAdd': {
      keyCode: 107,
      code: 'NumpadAdd',
      key: '+',
      location: 3
    },
    'NumpadSubtract': {
      keyCode: 109,
      code: 'NumpadSubtract',
      key: '-',
      location: 3
    },
    'NumpadDivide': {
      keyCode: 111,
      code: 'NumpadDivide',
      key: '/',
      location: 3
    },
    'F1': {
      keyCode: 112,
      code: 'F1',
      key: 'F1'
    },
    'F2': {
      keyCode: 113,
      code: 'F2',
      key: 'F2'
    },
    'F3': {
      keyCode: 114,
      code: 'F3',
      key: 'F3'
    },
    'F4': {
      keyCode: 115,
      code: 'F4',
      key: 'F4'
    },
    'F5': {
      keyCode: 116,
      code: 'F5',
      key: 'F5'
    },
    'F6': {
      keyCode: 117,
      code: 'F6',
      key: 'F6'
    },
    'F7': {
      keyCode: 118,
      code: 'F7',
      key: 'F7'
    },
    'F8': {
      keyCode: 119,
      code: 'F8',
      key: 'F8'
    },
    'F9': {
      keyCode: 120,
      code: 'F9',
      key: 'F9'
    },
    'F10': {
      keyCode: 121,
      code: 'F10',
      key: 'F10'
    },
    'F11': {
      keyCode: 122,
      code: 'F11',
      key: 'F11'
    },
    'F12': {
      keyCode: 123,
      code: 'F12',
      key: 'F12'
    },
    'F13': {
      keyCode: 124,
      code: 'F13',
      key: 'F13'
    },
    'F14': {
      keyCode: 125,
      code: 'F14',
      key: 'F14'
    },
    'F15': {
      keyCode: 126,
      code: 'F15',
      key: 'F15'
    },
    'F16': {
      keyCode: 127,
      code: 'F16',
      key: 'F16'
    },
    'F17': {
      keyCode: 128,
      code: 'F17',
      key: 'F17'
    },
    'F18': {
      keyCode: 129,
      code: 'F18',
      key: 'F18'
    },
    'F19': {
      keyCode: 130,
      code: 'F19',
      key: 'F19'
    },
    'F20': {
      keyCode: 131,
      code: 'F20',
      key: 'F20'
    },
    'F21': {
      keyCode: 132,
      code: 'F21',
      key: 'F21'
    },
    'F22': {
      keyCode: 133,
      code: 'F22',
      key: 'F22'
    },
    'F23': {
      keyCode: 134,
      code: 'F23',
      key: 'F23'
    },
    'F24': {
      keyCode: 135,
      code: 'F24',
      key: 'F24'
    },
    'NumLock': {
      keyCode: 144,
      code: 'NumLock',
      key: 'NumLock'
    },
    'ScrollLock': {
      keyCode: 145,
      code: 'ScrollLock',
      key: 'ScrollLock'
    },
    'AudioVolumeMute': {
      keyCode: 173,
      code: 'AudioVolumeMute',
      key: 'AudioVolumeMute'
    },
    'AudioVolumeDown': {
      keyCode: 174,
      code: 'AudioVolumeDown',
      key: 'AudioVolumeDown'
    },
    'AudioVolumeUp': {
      keyCode: 175,
      code: 'AudioVolumeUp',
      key: 'AudioVolumeUp'
    },
    'MediaTrackNext': {
      keyCode: 176,
      code: 'MediaTrackNext',
      key: 'MediaTrackNext'
    },
    'MediaTrackPrevious': {
      keyCode: 177,
      code: 'MediaTrackPrevious',
      key: 'MediaTrackPrevious'
    },
    'MediaStop': {
      keyCode: 178,
      code: 'MediaStop',
      key: 'MediaStop'
    },
    'MediaPlayPause': {
      keyCode: 179,
      code: 'MediaPlayPause',
      key: 'MediaPlayPause'
    },
    'Semicolon': {
      keyCode: 186,
      code: 'Semicolon',
      shiftKey: ':',
      key: ';'
    },
    'Equal': {
      keyCode: 187,
      code: 'Equal',
      shiftKey: '+',
      key: '='
    },
    'NumpadEqual': {
      keyCode: 187,
      code: 'NumpadEqual',
      key: '=',
      location: 3
    },
    'Comma': {
      keyCode: 188,
      code: 'Comma',
      shiftKey: '<',
      key: ','
    },
    'Minus': {
      keyCode: 189,
      code: 'Minus',
      shiftKey: '_',
      key: '-'
    },
    'Period': {
      keyCode: 190,
      code: 'Period',
      shiftKey: '>',
      key: '.'
    },
    'Slash': {
      keyCode: 191,
      code: 'Slash',
      shiftKey: '?',
      key: '/'
    },
    'Backquote': {
      keyCode: 192,
      code: 'Backquote',
      shiftKey: '~',
      key: '`'
    },
    'BracketLeft': {
      keyCode: 219,
      code: 'BracketLeft',
      shiftKey: '{ ',
      key: '['
    },
    'Backslash': {
      keyCode: 220,
      code: 'Backslash',
      shiftKey: '|',
      key: '\\'
    },
    'BracketRight': {
      keyCode: 221,
      code: 'BracketRight',
      shiftKey: ' }',
      key: ']'
    },
    'Quote': {
      keyCode: 222,
      code: 'Quote',
      shiftKey: '"',
      key: '\''
    },
    'AltGraph': {
      keyCode: 225,
      code: 'AltGraph',
      key: 'AltGraph'
    },
    'Props': {
      keyCode: 247,
      code: 'Props',
      key: 'CrSel'
    },
    'Cancel': {
      keyCode: 3,
      key: 'Cancel',
      code: 'Abort'
    },
    'Clear': {
      keyCode: 12,
      key: 'Clear',
      code: 'Numpad5',
      location: 3
    },
    'Shift': {
      keyCode: 16,
      key: 'Shift',
      code: 'ShiftLeft',
      location: 1
    },
    'Control': {
      keyCode: 17,
      key: 'Control',
      code: 'ControlLeft',
      location: 1
    },
    'Alt': {
      keyCode: 18,
      key: 'Alt',
      code: 'AltLeft',
      location: 1
    },
    'Accept': {
      keyCode: 30,
      key: 'Accept'
    },
    'ModeChange': {
      keyCode: 31,
      key: 'ModeChange'
    },
    ' ': {
      keyCode: 32,
      key: ' ',
      code: 'Space'
    },
    'Print': {
      keyCode: 42,
      key: 'Print'
    },
    'Execute': {
      keyCode: 43,
      key: 'Execute',
      code: 'Open'
    },
    '\u0000': {
      keyCode: 46,
      key: '\u0000',
      code: 'NumpadDecimal',
      location: 3
    },
    'a': {
      keyCode: 65,
      key: 'a',
      code: 'KeyA'
    },
    'b': {
      keyCode: 66,
      key: 'b',
      code: 'KeyB'
    },
    'c': {
      keyCode: 67,
      key: 'c',
      code: 'KeyC'
    },
    'd': {
      keyCode: 68,
      key: 'd',
      code: 'KeyD'
    },
    'e': {
      keyCode: 69,
      key: 'e',
      code: 'KeyE'
    },
    'f': {
      keyCode: 70,
      key: 'f',
      code: 'KeyF'
    },
    'g': {
      keyCode: 71,
      key: 'g',
      code: 'KeyG'
    },
    'h': {
      keyCode: 72,
      key: 'h',
      code: 'KeyH'
    },
    'i': {
      keyCode: 73,
      key: 'i',
      code: 'KeyI'
    },
    'j': {
      keyCode: 74,
      key: 'j',
      code: 'KeyJ'
    },
    'k': {
      keyCode: 75,
      key: 'k',
      code: 'KeyK'
    },
    'l': {
      keyCode: 76,
      key: 'l',
      code: 'KeyL'
    },
    'm': {
      keyCode: 77,
      key: 'm',
      code: 'KeyM'
    },
    'n': {
      keyCode: 78,
      key: 'n',
      code: 'KeyN'
    },
    'o': {
      keyCode: 79,
      key: 'o',
      code: 'KeyO'
    },
    'p': {
      keyCode: 80,
      key: 'p',
      code: 'KeyP'
    },
    'q': {
      keyCode: 81,
      key: 'q',
      code: 'KeyQ'
    },
    'r': {
      keyCode: 82,
      key: 'r',
      code: 'KeyR'
    },
    's': {
      keyCode: 83,
      key: 's',
      code: 'KeyS'
    },
    't': {
      keyCode: 84,
      key: 't',
      code: 'KeyT'
    },
    'u': {
      keyCode: 85,
      key: 'u',
      code: 'KeyU'
    },
    'v': {
      keyCode: 86,
      key: 'v',
      code: 'KeyV'
    },
    'w': {
      keyCode: 87,
      key: 'w',
      code: 'KeyW'
    },
    'x': {
      keyCode: 88,
      key: 'x',
      code: 'KeyX'
    },
    'y': {
      keyCode: 89,
      key: 'y',
      code: 'KeyY'
    },
    'z': {
      keyCode: 90,
      key: 'z',
      code: 'KeyZ'
    },
    'Meta': {
      keyCode: 91,
      key: 'Meta',
      code: 'MetaLeft',
      location: 1
    },
    '*': {
      keyCode: 106,
      key: '*',
      code: 'NumpadMultiply',
      location: 3
    },
    '+': {
      keyCode: 107,
      key: '+',
      code: 'NumpadAdd',
      location: 3
    },
    '-': {
      keyCode: 109,
      key: '-',
      code: 'NumpadSubtract',
      location: 3
    },
    '/': {
      keyCode: 111,
      key: '/',
      code: 'NumpadDivide',
      location: 3
    },
    ';': {
      keyCode: 186,
      key: ';',
      code: 'Semicolon'
    },
    '=': {
      keyCode: 187,
      key: '=',
      code: 'Equal'
    },
    ',': {
      keyCode: 188,
      key: ',',
      code: 'Comma'
    },
    '.': {
      keyCode: 190,
      key: '.',
      code: 'Period'
    },
    '`': {
      keyCode: 192,
      key: '`',
      code: 'Backquote'
    },
    '[': {
      keyCode: 219,
      key: '[',
      code: 'BracketLeft'
    },
    '\\': {
      keyCode: 220,
      key: '\\',
      code: 'Backslash'
    },
    ']': {
      keyCode: 221,
      key: ']',
      code: 'BracketRight'
    },
    '\'': {
      keyCode: 222,
      key: '\'',
      code: 'Quote'
    },
    'Attn': {
      keyCode: 246,
      key: 'Attn'
    },
    'CrSel': {
      keyCode: 247,
      key: 'CrSel',
      code: 'Props'
    },
    'ExSel': {
      keyCode: 248,
      key: 'ExSel'
    },
    'EraseEof': {
      keyCode: 249,
      key: 'EraseEof'
    },
    'Play': {
      keyCode: 250,
      key: 'Play'
    },
    'ZoomOut': {
      keyCode: 251,
      key: 'ZoomOut'
    },
    ')': {
      keyCode: 48,
      key: ')',
      code: 'Digit0'
    },
    '!': {
      keyCode: 49,
      key: '!',
      code: 'Digit1'
    },
    '@': {
      keyCode: 50,
      key: '@',
      code: 'Digit2'
    },
    '#': {
      keyCode: 51,
      key: '#',
      code: 'Digit3'
    },
    '$': {
      keyCode: 52,
      key: '$',
      code: 'Digit4'
    },
    '%': {
      keyCode: 53,
      key: '%',
      code: 'Digit5'
    },
    '^': {
      keyCode: 54,
      key: '^',
      code: 'Digit6'
    },
    '&': {
      keyCode: 55,
      key: '&',
      code: 'Digit7'
    },
    '(': {
      keyCode: 57,
      key: '(',
      code: 'Digit9'
    },
    'A': {
      keyCode: 65,
      key: 'A',
      code: 'KeyA'
    },
    'B': {
      keyCode: 66,
      key: 'B',
      code: 'KeyB'
    },
    'C': {
      keyCode: 67,
      key: 'C',
      code: 'KeyC'
    },
    'D': {
      keyCode: 68,
      key: 'D',
      code: 'KeyD'
    },
    'E': {
      keyCode: 69,
      key: 'E',
      code: 'KeyE'
    },
    'F': {
      keyCode: 70,
      key: 'F',
      code: 'KeyF'
    },
    'G': {
      keyCode: 71,
      key: 'G',
      code: 'KeyG'
    },
    'H': {
      keyCode: 72,
      key: 'H',
      code: 'KeyH'
    },
    'I': {
      keyCode: 73,
      key: 'I',
      code: 'KeyI'
    },
    'J': {
      keyCode: 74,
      key: 'J',
      code: 'KeyJ'
    },
    'K': {
      keyCode: 75,
      key: 'K',
      code: 'KeyK'
    },
    'L': {
      keyCode: 76,
      key: 'L',
      code: 'KeyL'
    },
    'M': {
      keyCode: 77,
      key: 'M',
      code: 'KeyM'
    },
    'N': {
      keyCode: 78,
      key: 'N',
      code: 'KeyN'
    },
    'O': {
      keyCode: 79,
      key: 'O',
      code: 'KeyO'
    },
    'P': {
      keyCode: 80,
      key: 'P',
      code: 'KeyP'
    },
    'Q': {
      keyCode: 81,
      key: 'Q',
      code: 'KeyQ'
    },
    'R': {
      keyCode: 82,
      key: 'R',
      code: 'KeyR'
    },
    'S': {
      keyCode: 83,
      key: 'S',
      code: 'KeyS'
    },
    'T': {
      keyCode: 84,
      key: 'T',
      code: 'KeyT'
    },
    'U': {
      keyCode: 85,
      key: 'U',
      code: 'KeyU'
    },
    'V': {
      keyCode: 86,
      key: 'V',
      code: 'KeyV'
    },
    'W': {
      keyCode: 87,
      key: 'W',
      code: 'KeyW'
    },
    'X': {
      keyCode: 88,
      key: 'X',
      code: 'KeyX'
    },
    'Y': {
      keyCode: 89,
      key: 'Y',
      code: 'KeyY'
    },
    'Z': {
      keyCode: 90,
      key: 'Z',
      code: 'KeyZ'
    },
    ':': {
      keyCode: 186,
      key: ':',
      code: 'Semicolon'
    },
    '<': {
      keyCode: 188,
      key: '<',
      code: 'Comma'
    },
    '_': {
      keyCode: 189,
      key: '_',
      code: 'Minus'
    },
    '>': {
      keyCode: 190,
      key: '>',
      code: 'Period'
    },
    '?': {
      keyCode: 191,
      key: '?',
      code: 'Slash'
    },
    '~': {
      keyCode: 192,
      key: '~',
      code: 'Backquote'
    },
    '{': {
      keyCode: 219,
      key: '{',
      code: 'BracketLeft'
    },
    '|': {
      keyCode: 220,
      key: '|',
      code: 'Backslash'
    },
    '}': {
      keyCode: 221,
      key: '}',
      code: 'BracketRight'
    },
    '"': {
      keyCode: 222,
      key: '"',
      code: 'Quote'
    }
  });

  function getSelectedTextRange($) {
    let win = $.ownerDocument.defaultView;
    let sel = win.getSelection();
    let start, end;
    if ('selectionStart' in $) {
      ({
        selectionStart: start,
        selectionEnd: end
      } = $);
    } else if (sel.containsNode($, true) || $.contains(sel.anchorNode) || $.contains(sel.focusNode)) {
      let range = sel.getRangeAt(0);
      let caretRange = range.cloneRange();
      caretRange.selectNodeContents($);
      caretRange.setEnd(range.startContainer, range.startOffset);
      start = caretRange.toString().length;
      caretRange.setEnd(range.endContainer, range.endOffset);
      end = caretRange.toString().length;
    }
    if (start || end) return [start, end];
  }
  const input = _ref => {
    let {
      text,
      event,
      replace,
      range
    } = _ref;
    return function input(_ref2) {
      let {
        i,
        $
      } = _ref2;
      return function* () {
        event = text ? {
          charCode: text.charCodeAt(0),
          ...event
        } : event;
        range &&= [].concat(range);
        let isInput = /^(input|textarea)$/i.test($.tagName);
        if (!isInput && !$.isContentEditable) return;
        let descr = isInput && Object.getOwnPropertyDescriptor($, 'value');
        if (descr) delete $.value;
        let canceled = !(yield i.trigger('beforeinput', event));
        if (!canceled) {
          let value = isInput ? $.value : $.textContent;
          if (replace) range ??= [0, value.length];
          range = [].concat(range ?? getSelectedTextRange($) ?? value.length);
          if (range.length === 1) range = [range[0], range[0]];
          if (event.key === 'Backspace') range[0] -= 1;
          if (event.key === 'Delete') range[1] += 1;
          value = value.slice(0, range[0]) + text + value.slice(range[1]);
          if (isInput) $.value = value;else $.textContent = value;
          yield i.trigger('input', {
            cancelable: false,
            ...event
          });
        }
        if (descr) Object.defineProperty($, 'value', descr);
      }();
    };
  };
  function* press(keys, options) {
    keys = typeof keys === 'string' ? keys.split('+') : keys;
    let {
      keyboard
    } = yield _ref3 => {
      let {
        set,
        keyboard
      } = _ref3;
      return set({
        immediate: {
          keyboard: new Keyboard(keyboard)
        }
      });
    };
    for (let key of keys) {
      let {
        text,
        event
      } = keyboard.parse(key);
      let canceled = !(yield this.trigger('keydown', event));
      if (!canceled && (text || event.key === 'Backspace' || event.key === 'Delete')) yield input({
          text,
          event,
          ...options
        });
      if (!keyboard.pressed(key)) keyboard.toggle(key);
    }
    if (!options?.hold) {
      for (let key of keys) {
        let {
          event
        } = keyboard.parse(key);
        yield this.trigger('keyup', {
          cancelable: false,
          ...event
        });
        keyboard.toggle(key);
      }
    }
  }

  function trigger(eventName) {
    let {
      bubbles = true,
      cancelable = true,
      ...eventProperties
    } = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return function* () {
      return yield _ref => {
        let {
          $
        } = _ref;
        return $.dispatchEvent(Object.assign(new Event(eventName, {
          bubbles,
          cancelable
        }), eventProperties));
      };
    }();
  }

  function* type(string, options) {
    yield this.focus();
    for (let i = 0; i < string.length; i++) {
      yield this.press(string[i], options);
      if (options?.delay && i < string.length - 1) yield new Promise(r => setTimeout(r, options?.delay));
    }
    if (yield _ref => {
      let {
        $
      } = _ref;
      return /input|textarea/i.test($.tagName);
    }) yield this.trigger('change', {
      cancelable: false
    });
    yield this.blur();
  }
  class TypeAction extends Interaction {
    constructor() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      super(_ref2 => {
        let {
          i
        } = _ref2;
        return type.apply(i, args);
      });
    }
    into(selector) {
      this[Context.Symbol].selector = selector;
      return this;
    }
  }

  var actions = /*#__PURE__*/Object.freeze({
    __proto__: null,
    blur: blur,
    click: click,
    focus: focus,
    press: press,
    trigger: trigger,
    type: TypeAction
  });

  class AttributeAssertion extends Assertion {
    constructor(name, expected) {
      super(_ref => {
        let {
          $
        } = _ref;
        return $.getAttribute(name) === expected;
      }, _ref2 => {
        let {
          $
        } = _ref2;
        return `#{this} \`${name}\` attribute is "${$.getAttribute(name)}" but expected "${expected}"`;
      }, `#{this} \`${name}\` attribute is "${expected}" but expected it not to be`);
    }
  }

  class CheckedAssertion extends Assertion {
    constructor() {
      super(_ref => {
        let {
          $
        } = _ref;
        return !!$.checked;
      }, '#{this} is not checked', '#{this} is checked');
    }
  }

  class DisabledAssertion extends Assertion {
    constructor() {
      super(_ref => {
        let {
          $
        } = _ref;
        return !!$.disabled;
      }, '#{this} is not disabled', '#{this} is disabled');
    }
  }

  class ExistsAssertion extends Assertion {
    constructor() {
      super(_ref => {
        let {
          $
        } = _ref;
        return !!$;
      }, 'Could not find #{this}', 'Found #{this} but expected not to');
    }
  }

  function documentFocusable(_ref) {
    let {
      $
    } = _ref;
    let {
      frameElement,
      document,
      self,
      top
    } = $.ownerDocument.defaultView;
    if (frameElement && !document.hasFocus() && self !== top) frameElement.focus();
    return document.hasFocus();
  }
  class FocusableAssertion extends Assertion {
    constructor() {
      function focusable(_ref2) {
        var _this = this;
        let {
          $
        } = _ref2;
        return function* () {
          yield _this.assert.not.disabled();
          yield _this.assert(documentFocusable, 'The document is not focusable');
          return !!(~$.tabIndex || $.isContentEditable);
        }();
      }
      super(focusable, '#{this} is not focusable', '#{this} is focusable');
    }
  }

  class FocusedAssertion extends Assertion {
    constructor() {
      super(_ref => {
        let {
          $
        } = _ref;
        return $ === globalThis.document?.activeElement;
      }, '#{this} is not focused', '#{this} is focused');
    }
  }

  class MatchesAssertion extends Assertion {
    constructor(selector) {
      super(_ref => {
        let {
          $
        } = _ref;
        return $.matches(selector);
      }, `#{this} does not match \`${selector}\``, `#{this} matches \`${selector}\``);
    }
  }

  class OverflowsAssertion extends Assertion {
    constructor(axis) {
      if (axis != null && axis !== 'x' && axis !== 'y') throw new Error(`Invalid overflow axis: ${axis}`);
      let overflowsX = _ref => {
        let {
          $
        } = _ref;
        return $.scrollWidth > $.clientWidth;
      };
      let overflowsY = _ref2 => {
        let {
          $
        } = _ref2;
        return $.scrollHeight > $.clientHeight;
      };
      let assertion = function* overflows() {
        if (axis === 'x') return yield overflowsX;
        if (axis === 'y') return yield overflowsY;
        return (yield overflowsY) || (yield overflowsX);
      };
      super(assertion, `#{this} has no overflow${axis ? `-${axis}` : ''}`, `#{this} has overflow${axis ? `-${axis}` : ''}`);
    }
  }

  function get(object, path) {
    return path.split('.').reduce((t, k) => t?.[k], object);
  }
  class PropertyAssertion extends Assertion {
    constructor(name, expected) {
      super(_ref => {
        let {
          $
        } = _ref;
        return get($, name) === expected;
      }, _ref2 => {
        let {
          $
        } = _ref2;
        return `#{this} \`${name}\` is \`${get($, name)}\` but expected \`${expected}\``;
      }, `#{this} \`${name}\` is \`${expected}\` but expected it not to be`);
    }
  }

  class ScrollableAssertion extends Assertion {
    constructor(axis) {
      if (axis != null && axis !== 'x' && axis !== 'y') throw new Error(`Invalid scroll axis: ${axis}`);
      let scrollableX = _ref => {
        let {
          $
        } = _ref;
        let {
          overflowX
        } = $.ownerDocument.defaultView.getComputedStyle($);
        return overflowX === 'scroll' || overflowX === 'auto';
      };
      let scrollableY = _ref2 => {
        let {
          $
        } = _ref2;
        let {
          overflowY
        } = $.ownerDocument.defaultView.getComputedStyle($);
        return overflowY === 'scroll' || overflowY === 'auto';
      };
      let assertion = function* scrollable() {
        yield this.assert.overflows(axis);
        if (axis === 'x') return yield scrollableX;
        if (axis === 'y') return yield scrollableY;
        return (yield scrollableY) || (yield scrollableX);
      };
      let dir = axis === 'x' ? 'horizontally' : 'vertically';
      super(assertion, `#{this} is not scrollable${axis ? ` ${dir}` : ''}`, `#{this} is scrollable${axis ? ` ${dir}` : ''}`);
    }
  }

  class SelectedAssertion extends Assertion {
    constructor() {
      super(_ref => {
        let {
          $
        } = _ref;
        return !!$.selected;
      }, '#{this} is not selected', '#{this} is selected');
    }
  }

  class TextAssertion extends Assertion {
    constructor(expected) {
      super(_ref => {
        let {
          $
        } = _ref;
        return $.innerText === expected;
      }, _ref2 => {
        let {
          $
        } = _ref2;
        return `#{this} text is "${$.innerText}" but expected "${expected}"`;
      }, `#{this} text is "${expected}" but expected it not to be`);
    }
  }

  class ValueAssertion extends Assertion {
    constructor(expected) {
      super(_ref => {
        let {
          $
        } = _ref;
        return $.value === expected;
      }, _ref2 => {
        let {
          $
        } = _ref2;
        return `#{this} value is "${$.value}" but expected "${expected}"`;
      }, `#{this} value is "${expected}" but expected it not to be`);
    }
  }

  function isHiddenByStyles(_ref) {
    let {
      $
    } = _ref;
    let {
      getComputedStyle
    } = $.ownerDocument.defaultView;
    let style = getComputedStyle($);
    return style.display === 'none' || style.visibility !== 'visible' || style.opacity < 0.1;
  }
  function isInsideViewport(_ref2) {
    let {
      $
    } = _ref2;
    let {
      top,
      left,
      bottom,
      right,
      width,
      height
    } = $.getBoundingClientRect();
    if (width === 0 || height === 0 || bottom < 0 || right < 0) return false;
    let {
      innerHeight,
      innerWidth
    } = $.ownerDocument.defaultView;
    return top < innerHeight && left < innerWidth;
  }
  function isElementAtPoint($, x, y) {
    let pointer = $.ownerDocument.elementFromPoint(x, y);
    if (pointer === $) return true;
    while (pointer = pointer?.parentNode) if (pointer === $) return true;
    return false;
  }
  function isElementOnTop(_ref3) {
    let {
      $
    } = _ref3;
    let {
      top,
      left,
      bottom,
      right,
      width,
      height
    } = $.getBoundingClientRect();
    let [cx, cy] = [left + width / 2, top + height / 2];
    return isElementAtPoint($, cx, cy) || isElementAtPoint($, cx, top + 1) || isElementAtPoint($, cx, bottom - 1) || isElementAtPoint($, left + 1, cy) || isElementAtPoint($, right - 1, cy);
  }
  class VisibleAssertion extends Assertion {
    constructor() {
      let assertion = function* visible() {
        if (yield isHiddenByStyles) return false;
        if (!(yield isInsideViewport)) return false;
        if (!(yield isElementOnTop)) return false;
        return true;
      };
      super(assertion, '#{this} is not visible', '#{this} is visible');
    }
  }

  var assertions = /*#__PURE__*/Object.freeze({
    __proto__: null,
    attribute: AttributeAssertion,
    checked: CheckedAssertion,
    disabled: DisabledAssertion,
    exists: ExistsAssertion,
    focusable: FocusableAssertion,
    focused: FocusedAssertion,
    matches: MatchesAssertion,
    overflows: OverflowsAssertion,
    property: PropertyAssertion,
    scrollable: ScrollableAssertion,
    selected: SelectedAssertion,
    text: TextAssertion,
    value: ValueAssertion,
    visible: VisibleAssertion
  });

  Interactor.defineActions(actions);
  Interactor.defineAssertions(assertions);
  const I = new Interactor();

  exports.Assert = Assert;
  exports.Assertion = Assertion;
  exports.Context = Context;
  exports.I = I;
  exports.Interaction = Interaction;
  exports.Interactor = Interactor;
  exports.default = I;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
